#
# Quick hack to convert TXT to SLC format required by Formulator(tm)
# implementation.  I'm guessing that a SLC file can contain only a
# single collection of levels (seeing as the XSD does not appear to
# be accessible), so we must generate one SLC for each collection in
# the .txt file.
#

 {
  my $ifn = $ARGV[0];

  die "Cannot open $ifn for reading\n"
    unless open( IFN, $ifn );

  my $ln;
  my $state = "get-title";
  my $title = "";
  my @maps;
  my @map;

  while ( defined( $ln = <IFN> ) ) {
    chomp $ln;
    if ( $state eq "get-title" ) {
      $title = $ln;

      my $ofn = &make_ofn( $title );

      die "Cannot open $ofn for writing\n"
	unless open( OFN, ">$ofn" );

      &process_title( $title );

      $state = "get-map";
      next;
    }

    if ( $state eq "get-map" ) {
      if ( $ln eq "%" ) {
	push( @maps, [ @map ] );
	@map = ();
	$state = "title-or-map-or-end";
	
      } else {
	push( @map, $ln );
      }
      next;
    }

    if ( $state eq "title-or-map-or-end" ) {
      if ( $ln =~ m/#/ ) {
	push( @map, $ln );
	$state = "get-map";

      } else {
	&process_ende( \@maps );
	close( OFN );
	
	if ( $ln eq "%" ) {
	  $state = "end";
	
	} else {
	  @maps = ();
	  @map = ();
	  $title = $ln;

	  my $ofn = &make_ofn( $title );

	  die "Cannot open $ofn for writing\n"
	    unless open( OFN, ">$ofn" );

	  &process_title( $title );

	  $state = "get-map";
	}
      }
      next;
    }

    if ( $state eq "end" ) {
      # we've reached the end... should be nothing more
      next;
    }
  }

  close( OFN );
  close( IFN );
  print "Done.\n";
 }



sub make_ofn ( $ ) {
  my( $title ) = @_;

  my $ofn = $title . ".slc";
  $ofn =~ s/ /_/g;

  return $ofn;
}

sub process_title ( $ ) {
  my( $title ) = @_;

  print OFN <<EOFTXT;
<?xml version="1.0" encoding="ISO-8859-1"?>
<SokobanLevels xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="SokobanLev.xsd">
EOFTXT

  print OFN "  <Title>$title</Title>\n";
  print OFN "  <Description>\n";
  print OFN "Translated Level collection\n";
  print OFN "  </Description>\n";
  print OFN "  <Email>foo@\localhost</Email>\n";
}

sub process_ende ( $ ) {
  my( $mapsref ) = @_;

  # scan mapsref to find max number rows and cols
  my( $maxrows, $maxcols ) = ( -1, -1 );

  for my $mref ( @$mapsref ) {
    $maxrows = $#$mref if ( $maxrows < $#$mref );
    for my $r ( @$mref ) {
      $maxcols = length( $r ) if ( $maxcols < length( $r ) );
    }
  }

  print OFN "  <LevelCollection Copyright=\"Translation\" MaxWidth=\""
    . $maxcols . "\" MaxHeight=\"" . (1+$maxrows) . "\">\n";

  my $level = 1;
  for my $mref ( @$mapsref ) {
    my $wid = -1;
    my $hei = 1 + $#$mref;

    for my $r ( @$mref ) {
      $wid = length( $r ) if ( $wid < length( $r ) );
    }

    print OFN "    <Level Id=\"$level\" Width=\"$wid\" Height=\"$hei\">\n";

    for my $r ( @$mref ) {
      print OFN "      <L>" . $r . "</L>\n";
    }

    $level++;
    print OFN "    </Level>\n";
  }

  print OFN "  </LevelCollection>\n";
  print OFN "</SokobanLevels>\n";
}
